"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const command_1 = require("@oclif/command");
const nodegit = require("nodegit");
const log_1 = require("../lib/log");
const rules_1 = require("../lib/rules");
class Scan extends command_1.Command {
    async run() {
        var _a;
        const { flags } = this.parse(Scan);
        const ctx = { summary: { findings: [] } };
        const check = async (entry, blobString, ruleType) => {
            const rule = rules_1.rules[ruleType];
            const matches = blobString.match(rule.pattern);
            if (matches && matches.length > 0) {
                log_1.default.debug(`[!] found ${matches.length} matches for ${ruleType}`);
                matches.forEach((matched) => {
                    blobString.split("\n").forEach((line, i) => {
                        var _a;
                        if (line.includes(matched)) {
                            const valuePart = flags.redact ? "" : `found ${matched} `;
                            const finding = {
                                ruleType,
                                detail: `${valuePart}in ${entry.toString()} on line ${i + 1}`,
                            };
                            (_a = ctx.summary) === null || _a === void 0 ? void 0 : _a.findings.push(finding);
                            log_1.default.warn(`[ ${finding.ruleType} ] ${finding.detail}`);
                        }
                    });
                });
            }
        };
        const testEntry = async (entry) => {
            var _a;
            log_1.default.debug(`checking ${entry.toString()}...`);
            let blob;
            try {
                blob = await entry.getBlob();
            }
            catch (e) {
                log_1.default.debug("no blob data");
                return;
            }
            try {
                const blobString = blob.toString();
                for (const ruleType in rules_1.rules) {
                    const filterEntry = (_a = rules_1.rules[ruleType]) === null || _a === void 0 ? void 0 : _a.filterEntry;
                    if (!filterEntry || filterEntry.test(entry.toString()))
                        await check(entry, blobString, ruleType);
                }
            }
            catch (e) {
                log_1.default.error(e);
            }
        };
        const scanRepoDir = async (dir) => {
            log_1.default.info(`scanning ${dir}`);
            try {
                const repo = await nodegit.Repository.open(path.resolve(dir));
                if (flags.pull) {
                    log_1.default.info("fetching from remotes...");
                    await repo.fetchAll({
                        callbacks: {
                            certificateCheck: () => 0,
                            credentials: (_url, user) => {
                                log_1.default.debug(`fetching from ${_url}`);
                                return nodegit.Cred.sshKeyFromAgent(user);
                            },
                        },
                    });
                }
                const commit = await repo.getMasterCommit();
                log_1.default.info(`analysing files at commit ${commit.toString()}`);
                const tree = await commit.getTree();
                for (const entry of tree.entries())
                    await testEntry(entry);
            }
            catch (e) {
                log_1.default.warn("directory not scanned - check debug log for details");
                log_1.default.debug(e);
            }
        };
        try {
            const resolvedDir = path.resolve(flags.dir || ".");
            // is this a git repo?
            const gitDir = path.resolve(resolvedDir, ".git");
            log_1.default.info(`SCAN STARTED: ${resolvedDir}`);
            log_1.default.debug(`checking for ${gitDir}`);
            if (!fs.existsSync(gitDir)) {
                // not a git repo, iterate directories
                log_1.default.debug(`${resolvedDir} is not a git repo, digging...`);
                const files = await fs.promises.readdir(resolvedDir);
                for (const file of files) {
                    const fullPath = path.resolve(resolvedDir, file);
                    const stat = await fs.promises.stat(fullPath);
                    if (stat.isDirectory())
                        await scanRepoDir(fullPath);
                }
            }
            else
                await scanRepoDir(resolvedDir);
        }
        catch (e) {
            log_1.default.error(e, "SCAN FAILED");
            this.exit(1);
        }
        log_1.default.info(`SCAN COMPLETE (found ${(_a = ctx.summary) === null || _a === void 0 ? void 0 : _a.findings.length} issues)`);
    }
}
exports.default = Scan;
Scan.description = "scans local git repositories for committed secrets";
Scan.examples = ["$ git-secret-scanner scan ~/code/github.com/my-org -r"];
Scan.flags = {
    help: command_1.flags.help({ char: "h" }),
    dir: command_1.flags.string({
        char: "d",
        description: "directory to scan (current direction if omitted)",
    }),
    pull: command_1.flags.boolean({ char: "p", description: "pull from repositories" }),
    redact: command_1.flags.boolean({
        char: "r",
        description: "redact all matched secret strings",
    }),
};
